@@ -1,8 +1,8 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*-
 # filename: totalopenstation-gui.py
-# Copyright 2008-2011 Stefano Costa <steko@iosa.it>
-# Copyright 2010 Luca Bianconi <luxetluc@yahoo.it>
+# Copyright 2008-2014 Stefano Costa <steko@iosa.it>
+# Copyright 2010,2012 Luca Bianconi <luxetluc@yahoo.it>
 #
 # This file is part of Total Open Station.
 #
@@ -31,6 +31,8 @@ from tkMessageBox import showwarning, showinfo, askokcancel
 import tkSimpleDialog
 import tkFileDialog
 
+import totalopenstation
+
 from totalopenstation.models import BUILTIN_MODELS
 from totalopenstation.formats import BUILTIN_INPUT_FORMATS
 from totalopenstation.output import BUILTIN_OUTPUT_FORMATS
@@ -115,14 +117,15 @@ class StatusBar(Frame):
 class AboutDialog(tkSimpleDialog.Dialog):
 
     def body(self, master):
-        title = "Total Open Station 0.2"
+        title = "Total Open Station %s" % totalopenstation.__version__
         message = _("""
-Total Open Station is copyright 2008-2010 Luca Bianconi, Stefano Costa
-and the IOSA project.
+Total Open Station is copyright 2008-2015 Luca Bianconi, Stefano Costa
+and the IOSA project, under the GNU GPL v3 or any later version.
 
-http://tops.berlios.de/
+http://tops.iosa.it/
 
-The application logo is copyright 2008 Lapo Calamandrei.""")
+The application logo is copyright 2008 Lapo Calamandrei under the same
+license.""")
 
         self.logo_data = logo_data
         self.logo = PhotoImage(data=self.logo_data)
@@ -305,6 +308,10 @@ class ProcessDialog(tkSimpleDialog.Dialog):
             sd_file.write(output.process())
             sd_file.close()
 
+class PreferencesDialog(tkSimpleDialog.Dialog):
+    '''A dialog to change preferences and options.'''
+
+
 
 class ErrorDialog(tkSimpleDialog.Dialog):
 
@@ -357,6 +364,33 @@ class Tops:
         self.main_frame = Frame(parent) ###
         self.main_frame.pack(expand=YES, fill=BOTH)
 
+		#MENU
+        self.menubar = Menu(self.myParent)
+        self.myParent.config(menu=self.menubar)
+
+        topsmenu = Menu(self.menubar, tearoff=0)
+        topsmenu.add_command(label=_("Connect"), command=self.connect)
+        topsmenu.add_command(label=_("Process data"), command=self.process)
+        topsmenu.add_separator()
+        topsmenu.add_command(label=_("Quit"), command=self.on_app_close)
+
+        filemenu = Menu(self.menubar, tearoff=0)
+        filemenu.add_command(label=_("Open file"), command=self.open_a_file)
+        filemenu.add_command(label=_("Save raw data"), command=self.save_a_file)
+
+        helpmenu = Menu(self.menubar, tearoff=0)
+        helpmenu.add_command(label=_("About TOPS"), command=self.about)
+
+        languagemenu = Menu(self.menubar, tearoff=0)
+        language = StringVar()
+        languagemenu.add_radiobutton(label="English",variable=language,value="en")
+        languagemenu.add_radiobutton(label="Italiano",variable=language,value="it")
+
+        self.menubar.add_cascade(label="Total Station", menu=topsmenu)
+        self.menubar.add_cascade(label="File", menu=filemenu)
+        #self.menubar.add_cascade(label="Language", menu=languagemenu)
+        self.menubar.add_cascade(label="Help", menu=helpmenu)
+
         self.upper_frame = Frame(self.main_frame) ###
         self.upper_frame.pack(side=TOP, expand=NO, padx=10,
                                    pady=5, ipadx=5, ipady=5)
@@ -397,7 +431,7 @@ class Tops:
 
         self.option1_label = Label(self.option1_frame,
                                    text=_("Port"),
-                                   width=25)
+                                   width=30)
         self.option1_label.pack(side=LEFT)
         self.option1_value = StringVar()
         self.option1_value.set(self.upref.getvalue('port'))
@@ -406,7 +440,7 @@ class Tops:
 #
         self.option1_entry = Entry(self.option1_frame,
                                    textvariable=self.option1_value,
-                                   width=25)
+                                   width=20)
 
 # ... comment out this Menubutton if you want to use the scan() output
 #
@@ -436,7 +470,7 @@ class Tops:
         self.optionMODEL_label = Label(self.optionMODEL_frame,
                                    text=_("Total Station"),
                                    justify=LEFT,
-                                   width=25)
+                                   width=30)
         self.optionMODEL_label.pack(side=LEFT, anchor=E)
         self.optionMODEL_value = StringVar()
 
@@ -446,7 +480,7 @@ class Tops:
                                         text="choose a model",
                                         textvariable=self.optionMODEL_value,
                                         relief=RAISED,
-                                        width=24)
+                                        width=20)
         self.optionMODEL_entry.menu = Menu(self.optionMODEL_entry, tearoff=0)
         self.optionMODEL_entry["menu"] = self.optionMODEL_entry.menu
 
@@ -464,7 +498,7 @@ class Tops:
 
         self.option2_label = Label(self.option2_frame,
                                    text="Baudrate",
-                                   width=25)
+                                   width=30)
         self.option2_label.pack(side=LEFT)
         self.option2_value = IntVar()
 
@@ -474,13 +508,13 @@ class Tops:
         except (AttributeError, AssertionError):
             self.option2_entry = Entry(self.option2_frame,
                                        textvariable=self.option2_value,
-                                       width=25)
+                                       width=20)
         else:
             self.option2_entry = Menubutton(self.option2_frame,
                                             text=_("choose a value"),
                                             textvariable=self.option2_value,
                                             relief=RAISED,
-                                            width=24)
+                                            width=20)
             self.option2_entry.menu = Menu(self.option2_entry, tearoff=0)
             self.option2_entry["menu"] = self.option2_entry.menu
             for key in sorted(serial.baudrate_constants.keys()): # dynamic list
@@ -498,14 +532,14 @@ class Tops:
         self.option3_label = Label(self.option3_frame,
                                    text=_("Bytesize"),
                                    justify=LEFT,
-                                   width=25)
+                                   width=30)
         self.option3_label.pack(side=LEFT, anchor=E)
         self.option3_value = IntVar()
         self.option3_entry = Menubutton(self.option3_frame,
                                         text=_("choose a value"),
                                         textvariable=self.option3_value,
                                         relief=RAISED,
-                                        width=24)
+                                        width=20)
         self.option3_entry.menu = Menu(self.option3_entry, tearoff=0)
         self.option3_entry["menu"] = self.option3_entry.menu
         for v in [8,7,6,5]:
@@ -521,14 +555,14 @@ class Tops:
         self.option4_label = Label(self.option4_frame,
                                    text=_("Parity setting"),
                                    justify=LEFT,
-                                   width=25)
+                                   width=30)
         self.option4_label.pack(side=LEFT, anchor=E)
         self.option4_value = StringVar()
         self.option4_entry = Menubutton(self.option4_frame,
                                         text=_("choose a value"),
                                         textvariable=self.option4_value,
                                         relief=RAISED,
-                                        width=24)
+                                        width=20)
         self.option4_entry.menu = Menu(self.option4_entry, tearoff=0)
         self.option4_entry["menu"] = self.option4_entry.menu
         for v in ['Even', 'None', 'Odd']:
@@ -544,14 +578,14 @@ class Tops:
         self.option5_label = Label(self.option5_frame,
                                    text=_("Stop bit"),
                                    justify=LEFT,
-                                   width=25)
+                                   width=30)
         self.option5_label.pack(side=LEFT, anchor=E)
         self.option5_value = IntVar()
         self.option5_entry = Menubutton(self.option5_frame,
                                         text=_("choose a value"),
                                         textvariable=self.option5_value,
                                         relief=RAISED,
-                                        width=24)
+                                        width=20)
         self.option5_entry.menu = Menu(self.option5_entry, tearoff=0)
         self.option5_entry["menu"] = self.option5_entry.menu
         for v in [1, 2]:
@@ -560,26 +594,34 @@ class Tops:
                                                     value=v)
         self.option5_entry.pack(side=LEFT, anchor=W)
 
+        # option 6 : time lapse between data packets
+        self.option6_frame = Frame(self.control_panel, relief=RIDGE, bd=1)
+        self.option6_frame.pack(side=TOP)
+
+        self.option6_label = Label(self.option6_frame,
+                                   text=_("Time lapse between data packets"),
+                                   justify=LEFT,
+                                   width=30)
+        self.option6_label.pack(side=LEFT, anchor=E)
+        self.option6_value = DoubleVar()
+        self.option6_value.set(self.upref.getvalue('sleeptime'))
+        self.option6_entry = Entry(self.option6_frame,
+                                   textvariable=self.option6_value,
+                                   relief=RAISED,
+                                   justify=RIGHT,
+                                   width=20)
+        self.option6_entry.pack(side=LEFT, anchor=W)
 
         # control buttons
 
         self.connect_button = Button(self.buttons_frame,
                                       text=_("Connect"),
-                                      background="green",
                                       padx=imb_buttonx,
                                       pady=imb_buttony)
         self.connect_button.pack(side=LEFT, anchor=S)
         self.connect_button.bind("<Button-1>", self.connect_action)
         self.connect_button.bind("<Return>", self.connect_action)
 
-        self.open_button = Button(self.buttons_frame,
-                                      text=_("Open file"),
-                                      padx=imb_buttonx,
-                                      pady=imb_buttony)
-        self.open_button.pack(side=LEFT, anchor=S)
-        self.open_button.bind("<Button-1>", self.open_action)
-        self.open_button.bind("<Return>", self.open_action)
-
         self.save_button = Button(self.buttons_frame,
                                       text=_("Save raw data"),
                                       padx=imb_buttonx,
@@ -590,29 +632,12 @@ class Tops:
 
         self.process_button = Button(self.buttons_frame,
                                      text=_("Process data"),
-                                     background="cyan",
                                      padx=imb_buttonx,
                                      pady=imb_buttony)
         self.process_button.pack(side=LEFT, anchor=S)
         self.process_button.bind("<Button-1>", self.process_action)
         self.process_button.bind("<Return>", self.process_action)
 
-        self.about_button = Button(self.buttons_frame,
-                                   text=_("About TOPS"),
-                                   padx=imb_buttonx,
-                                   pady=imb_buttony)
-        self.about_button.pack(side=LEFT, anchor=S)
-        self.about_button.bind("<Button-1>", self.about_action)
-        self.about_button.bind("<Return>", self.about_action)
-
-        self.exit_button = Button(self.buttons_frame,
-                                  text=_("Quit"),
-                                  padx=imb_buttonx,
-                                  pady=imb_buttony)
-        self.exit_button.pack(side=LEFT, anchor=S)
-        self.exit_button.bind("<Button-1>", self.exit_action)
-        self.exit_button.bind("<Return>", self.exit_action)
-
         self.status = StatusBar(self.main_frame)
         self.status.set('Welcome to Total Open Station')
         self.status.pack(side=BOTTOM, fill=X)
@@ -631,9 +656,26 @@ class Tops:
         self.text_area['yscrollcommand'] = self.scrollY.set
         self.scrollY.pack(side=RIGHT, expand=YES, fill=Y, anchor=W)
 
+        # init stuff
+        self.myParent.title("Total Open Station")
+        self.myParent.protocol("WM_DELETE_WINDOW", self.on_app_close)
+        self.print_model()
+        self.myParent.mainloop()
+
+    def on_click_language(self):
+        '''
+            open select language dialog
+        '''
+        pass
+
+    def on_app_close(self):
+        '''Callback function to ask confirmation before quitting the application.'''
+
+        if askokcancel("Quit","Do you really want to quit application ?"):
+            self.myParent.destroy()
 
     def exit_action(self, event):
-        self.myParent.destroy()
+        self.on_app_close()
 
     def print_model(self):
         model = self.optionMODEL_value.get()
@@ -654,7 +696,7 @@ class Tops:
                                     ipadx=5,
                                     ipady=5)
 
-    def connect_action(self, event):
+    def connect(self):
 
         chosen_model = self.optionMODEL_value.get()
         chosen_port = self.option1_value.get()
@@ -665,8 +707,7 @@ class Tops:
             if chosen_model == 'custom':
 
                 # dictionary for passing options to Serial
-                self.options = {'port': self.option1_value.get(),
-                                'baudrate': self.option2_value.get(),
+                self.options = {'baudrate': self.option2_value.get(),
                                 'bytesize': self.option3_value.get(),
                                 'parity': self.option4_value.get(),
                                 'stopbits': self.option5_value.get()}
@@ -674,7 +715,7 @@ class Tops:
                 self.options = {}
 
             modelclass = BUILTIN_MODELS[chosen_model]
-            if isinstance(outputclass, tuple):
+            if isinstance(modelclass, tuple):
                 try:
                     # builtin model builder
                     mod, cls, name = modelclass
@@ -683,32 +724,40 @@ class Tops:
                 except ImportError, msg:
                     showwarning(_('Import error'),
                                 _('Error loading the required model module: %s' % msg))
-                mc = modelclass(chosen_port)
+
+                mc = modelclass(chosen_port, **self.options)
+
                 try:
+                    mc.close()  # sometimes the port will be already open for no reason
                     mc.open()
                 except serial.SerialException, detail:
                     e = ErrorDialog(self.myParent, detail)
                 else:
                     st = DownloadDialog(self.myParent)
+                    sleeptime = float(self.option6_value.get())
                     if st.result:
                         self.status.set(_("Waiting for data: please start transfer from your total station menu."))
                         while mc.inWaiting() == 0:
-                            sleep(0.1)
+                            sleep(sleeptime)
                         n = mc.inWaiting()
                         result = mc.read(n)
-                        self.replace_text(result)
-                        sleep(0.1)
+                        self.replace_text(str(result))
+                        sleep(sleeptime)
+
                         while mc.inWaiting() > 0:
                             newdata = mc.read(mc.inWaiting())
                             result += newdata
                             self.status.set(_('Downloaded %d bytes'), len(result))
-                            self.replace_text(result)
-                            sleep(0.3) # TODO determine sleep time from baudrate
+                            self.replace_text(str(result))
+                            sleep(sleeptime) # TODO determine sleep time from baudrate
                         mc.close()
                         showinfo(_('Success!'),
                                  _('Download finished!\nYou have %d bytes of data.') % len(result))
 
-    def open_action(self, event):
+    def connect_action(self, event):
+        self.connect()
+
+    def open_a_file(self):
         try:
             d = tkFileDialog.askopenfilename()
             of = open(d, 'r')
@@ -717,11 +766,17 @@ class Tops:
         except:
             pass
 
-    def process_action(self, event):
+    def open_action(self, event):
+        self.open_a_file()
+
+    def process(self):
         data = self.text_area.get("1.0", END)
         d = ProcessDialog(self.myParent, data)
 
-    def save_action(self, event):
+    def process_action(self, event):
+        self.process()
+
+    def save_a_file(self):
         try:
             sd = tkFileDialog.asksaveasfilename(defaultextension='.tops')
             data = self.text_area.get("1.0", END)
@@ -730,9 +785,15 @@ class Tops:
         except:
             pass
 
-    def about_action(self, event):
+    def save_action(self, event):
+        self.save_a_file()
+
+    def about(self):
         d = AboutDialog(self.myParent)
 
+    def about_action(self, event):
+        self.about()
+
     def replace_text(self, text):
         self.text_area.delete("1.0", END)
         self.text_area.insert(END, text.replace('\r', ''))
@@ -742,12 +803,12 @@ class Tops:
 
 root = Tk()
 Tops = Tops(root)
-root.title("Total Open Station")
-root.mainloop()
 
-#save user's preferences (model and port)
+
+#save user's preferences (model, port and sleeptime if custom model)
 
 atexit.register(Tops.upref.setvalues,
                 {'model': Tops.optionMODEL_value.get(),
-                 'port':Tops.option1_value.get()})
-
+                 'port': Tops.option1_value.get(),
+                 'sleeptime': Tops.option6_value.get(),
+                 })
